/**
- Export constructor to create instance of Database

@exports database
@author kevin leptons <kevin.leptons@gmail.com>
@example
const database = require('./lib/database')

var logger = flatlog({caller: 'gwisp'})
var db = database(url, null, null, caller)
*/
module.exports = function (url, username, password, logger) {
  return new Database(url, username, password, logger)
}

const async = require('async')
const mongodb = require('mongodb')

const MongoClient = mongodb.MongoClient

function Database(url, username, password, logger) {
  this._url = url
  this._username = username
  this._password = password
  this._logger = logger
  this._dbclient = null
}
const prototype = Database.prototype

prototype.connect = function (callback) {
  const self = this
  MongoClient.connect(this._url, function (err, db) {
    if (err) return callback(err)

    self._dbclient = db
    callback(err, db)
  })
}

prototype.close = function (callback) {
  const self = this

  if (!self._dbclient) return callback(null)

  self._dbclient.close(true, function (err) {
    if (err) return callback(err)
    self._dbclient = null

    callback(null)
  })
}

prototype.setup = function (assetDir, callback) {
  const self = this

  self._logger.info('drop database')
  self.drop(function (err) {
    if (err) {
      self._logger.error('drop database')
      return callback(err)
    }

    self.connect(function (err, db) {
      if (err) return callback(err)
      self._createCollections(require('./collections'), callback)
    })
  })
}

prototype.drop = function (callback) {
  const self = this

  self.connect(function (err, db) {
    if (err) return callback(err)

    db.dropDatabase(function (err, res) {
      if (err) return callback(err)

      self.close(callback)
    })
  })
}

prototype._createCollection = function (name, schema, index, callback) {
  const self = this

  self._dbclient.createCollection(name, schema, function (err) {
    if (err) return callback(err)

    self._dbclient.collection(name).createIndex(index.key, index.opt, function (err) {
      callback(err)
    })
  })
}

prototype._createCollections = function(collections, callback) {
  const self = this
  const calls = []

  self._logger.info('create collections')
  collections.forEach(function(item) {
    calls.push(function (callback) {
      self._createCollection(item.name, item.schema, item.index, callback)
    })
  })

  async.series(calls, function (err) {
    // todo: push sample data to database
    self._logger.info('push sample data to database')

    callback(err)
  })
}
