/**
flatlog module. it exports constructor of FlatLog

@exports flatlog
@author kevin leptons <kevin.leptons@gmail.com>
@example
const flatlog = requier('./lib/flatlog')

var config = {
  verbose: tue,
  'gwisp'
}
var logger = new flatlog(config)

logger.log(INFO, 'start new day')
logger.log(NOTICE, 'work done', 'stuart')
logger.log(ERROR, 'go to workspace')
*/
module.exports = function (config) {
  return new FlatLog(config)
}

const chalk = require('chalk')
const columnify = require('columnify')

const EMERGENCY = 0 // system is unusable
const ALERT = 1 // action must be taken immediately
const CRITICAL = 2 // the system is in critical condition
const ERROR = 3 // error condition
const WARNING = 4 // warning condition
const NOTICE = 5 // a normal but significant condition
const INFO = 6 // a purely informational message
const DEBUG = 7 // messages to debug an application

/**
Configuration of logger

@memberof module:flatlog
@typedef {object} Config
@property {boolean} [config.verbose=true] Decide log message or not
@property {string} [config.caller=anonymous] Default caller will be write
@example
var config = {
  verbose: true,
  'gwisp'
}
*/

/**
Create new FlatLog from configuration. It help write log message simply
and readable

@constructor
@memberof module:flatlog
@class FlatLog
@param {module:flatlog.Config} config - configuration of logger
@return {FlatLog} the new flat log object
@example
var logger = new FlatLog({caller: 'kevin'})
*/
function FlatLog (config) {
  this._config = config
  if (this._config.verbose === undefined) {
    this._config.verbose = true
  }
  if (!this._config.caller) {
    this._config.caller = chalk.red('anonymous')
  }
}
const prototype = FlatLog.prototype

/**
Log message to terminal by one line format

@memberof! module:flatlog.FlatLog
@param {int} level level of message
- 0 EMERGENCY system is unusable
- 1 ALERT action must be taken immediately
- 2 CRITICAL the system is in critical condition
- 3 ERROR error condition
- 4 WARNING warning condition
- 5 NOTICE a normal but significant condition
- 6 INFO a purely informational message
- 7 DEBUG messages to debug an application
@param {string} message message to write
@example
// see constructor for detail
var logger = new FlatLog(config)

logger.log(INFO, 'start new day')
logger.log(NOTICE, 'work done', 'stuart')
logger.log(ERROR, 'go to workspace')
*/
prototype.log = function (level, message) {
  const self = this

  // do not thing if configuration say not verbose
  if (!self._config.verbose) {
    return
  }

  // chose format for each log level
  // convert log level from integer to string
  var format
  var levelName
  switch (level) {
    case EMERGENCY:
      format = chalk.black
      levelName = 'EMERGENCY'
      break
    case ALERT:
      format = chalk.red
      levelName = 'ALERT'
      break
    case CRITICAL:
      format = chalk.green
      levelName = 'CRITICAL'
      break
    case ERROR:
      format = chalk.yellow
      levelName = 'ERROR'
      break
    case WARNING:
      format = chalk.blue
      levelName = 'WARNING'
      break
    case NOTICE:
      format = chalk.magenta
      levelName = 'NOTICE'
      break
    case INFO:
      format = chalk.cyan
      levelName = 'INFO'
      break
    case DEBUG:
      format = chalk.gray
      levelName = 'DEBUG'
      break
    default:
      throw new Error('log level is invalid')
  }

  // create data of message
  var data = [{
    level: format.bold(levelName),
    message: message,
    caller: self._config.caller
  }]

  // format data
  var line = columnify(data, {
    showHeaders: false,
    truncate: true,
    config: {
      level: {
        minWidth: 8,
        maxWidth: 8
      },
      message: {
        minWidth: 59,
        maxWidth: 59
      },
      caller: {
        minWidth: 10,
        maxWidth: 10,
        align: 'right'
      }
    }
  })

  console.log(line)
}

/**
Log message with EMERGENCY level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.emergency('system is unusable')
*/
prototype.emergency = function (message) {
  this.log(EMERGENCY, message)
}

/**
Log message with ALERT level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.alert('action must be taken immediately')
*/
prototype.alert = function (message) {
  this.log(ALERT, message)
}

/**
Log message with CRITICAL level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.critical('the system is in critical condition')
*/
prototype.critical = function (message) {
  this.log(CRITICAL, message)
}

/**
Log message with ERROR level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.error('error condition')
*/
prototype.error = function (message) {
  this.log(ERROR, message)
}

/**
Log message with WARNING level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.warning('warning condition')
*/
prototype.warning = function (message) {
  this.log(WARNING, message)
}

/**
Log message with NOTICE level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.notice('a normal but significant condition')
*/
prototype.notice = function (message) {
  this.log(NOTICE, message)
}

/**
Log message with INFO level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.info('a purely informational message')
*/
prototype.info = function (message) {
  this.log(INFO, message)
}

/**
Log message with DEBUG level

@memberof module:flatlog.FlatLog
@param {string} message log message
@example
// see constructor for detail
var logger = new Logger(config)

logger.debug('messages to debug an application')
*/
prototype.debug = function (message) {
  this.log(DEBUG, message)
}
